@@ -4,11 +4,13 @@ from py4j.java_gateway import get_field
 from planit import GatewayUtils
 from planit import GatewayState
 from planit import OutputType
-from planit import RouteIdType
+from planit import PathIdType
 from planit import OutputProperty
 from planit import PhysicalCost
 from planit import VirtualCost
 from planit import Smoothing
+from _decimal import Decimal
+from numpy import string_
 
 class BaseWrapper(object):
     """ Base wrapper class which always holds a java counter part instance and a generic way to pass on method calls to the encapsulated 
@@ -119,7 +121,7 @@ class AssignmentWrapper(BaseWrapper):
             elif output_type.value == 'PATH':
                 self._path_output_type_configuration = PathOutputTypeConfigurationWrapper(self._java_counterpart.activateOutput(output_type_instance))
             else:
-                raise ValueError("Attempted to deactivate unknown output type " + output_type.value)     
+                raise ValueError("Attempted to activate unknown output type " + output_type.value)     
     
     def deactivate_output(self, output_type: OutputType):   
         """Deactivate specified output type on the assignment
@@ -128,13 +130,13 @@ class AssignmentWrapper(BaseWrapper):
         output_type_instance = self.__create_java_output_type(output_type)
         if self.is_output_type_active(output_type_instance):
             if output_type.value == "LINK":
-                self._link_output_type_configuration = LinkOutputTypeConfigurationWrapper(self._java_counterpart.deactivateOutput(output_type_instance))
+                self._java_counterpart.deactivateOutput(output_type_instance)
             elif output_type.value == "OD":
-                self._origin_destination_output_type_configuration = OriginDestinationOutputTypeConfigurationWrapper(self._java_counterpart.deactivateOutput(output_type_instance))
+                self._java_counterpart.deactivateOutput(output_type_instance)
             elif output_type.value == 'PATH':
-                self._path_output_type_configuration = PathOutputTypeConfigurationWrapper(self._java_counterpart.deactivateOutput(output_type_instance))
+                self._java_counterpart.deactivateOutput(output_type_instance)
             else:
-                raise ValueError("Attempted to activate unknown output type " + output_type.value)     
+                raise ValueError("Attempted to de-activate unknown output type " + output_type.value)     
   
     @property
     def gap_function(self):
@@ -228,6 +230,13 @@ class LinkSegmentsWrapper(BaseWrapper):
     def __init__(self, java_counterpart):
         super().__init__(java_counterpart)
         
+class LinkSegmentTypesWrapper(BaseWrapper):
+    """ Wrapper around the Java LinkSegmentTypes class
+    """
+    
+    def __init__(self, java_counterpart):
+        super().__init__(java_counterpart)        
+        
 class LinkSegmentExpectedResultsDtoWrapper(BaseWrapper):
     """ Wrapper around the Java Link Segment Expected Results DTO class instance
     """
@@ -378,38 +387,48 @@ class BPRCostWrapper(PhysicalCostWrapper):
         self._network_instance = network_instance
         modes_counterpart = self._network_instance.field("modes")
         self._modes_instance = ModesWrapper(modes_counterpart)
-        link_segments_counterpart = self._network_instance.field("linkSegments")
-        self._link_segments_instance = LinkSegmentsWrapper(link_segments_counterpart)
         
-    def set_default_parameters(self, alpha, beta, mode_external_id=None, link_segment_type_external_id=None):
+        network_layers_counterpart = network_instance.field("infrastructureLayers")
+        self._layers_instance = ModesWrapper(network_layers_counterpart)        
+        
+        
+    def set_default_parameters(self, alpha: float, beta: float, mode_xml_id:str =None, link_segment_type_xml_id:str =None):
         """Set the default BPR functions parameters 
         :param alpha value of alpha parameter
         :param beta value of beta parameter
-        :param mode_external_id if included, default parameters only apply to this mode
-        :param link_segment_type_external_id if included, default parameters apply to this link segment type and mode
+        :param mode_xml_id if included, default parameters only apply to this mode
+        :param link_segment_type_xml_id if included, default parameters apply to this link segment type and mode
         """
-        if (mode_external_id == None):
+        if (mode_xml_id == None):
             self._java_counterpart.setDefaultParameters(alpha, beta)
         else:
-            mode_counterpart = self._modes_instance.get_mode_by_external_id(mode_external_id, True)
-            if (link_segment_type_external_id == None):
+            mode_counterpart = self._modes_instance.get_by_xml_id(mode_xml_id)
+            if (link_segment_type_xml_id == None):
                 self._java_counterpart.setDefaultParameters(mode_counterpart, alpha, beta)
             else:
-                link_segment_type_counterpart = self._network_instance.get_macroscopic_link_segment_type_by_external_id(link_segment_type_external_id, True)
-                link_segment_type_instance = MacroscopicLinkSegmentTypeWrapper(link_segment_type_counterpart)
-                self.setDefaultParameters(link_segment_type_instance.java, mode_counterpart, alpha, beta)
+                layer_counterpart = self._layers_instance.get(mode_counterpart)
+                # cannot use field "linkSegmentTypes" for some reason, likely because layer_counterpart is a InfrastructureLayer to py4j and it cannot access field of 
+                # derived type, hence we must call method for it to find the derived implementation
+                link_segment_types_counterpart = layer_counterpart.getLinkSegmentTypes()
+                link_segment_types_instance = LinkSegmentTypesWrapper(link_segment_types_counterpart)
                 
-    def set_parameters(self, alpha, beta, mode_external_id, link_segment_external_id):
+                link_segment_type_counterpart = link_segment_types_instance.get_by_xml_id(link_segment_type_xml_id)
+                self.setDefaultParameters(link_segment_type_counterpart, mode_counterpart, alpha, beta)
+                
+    def set_parameters(self, alpha: float, beta:float, mode_xml_id: str, link_segment_xml_id: str):
         """Set the default BPR functions parameters 
         :param alpha value of alpha parameter
         :param beta value of beta parameter
-        :param mode_external_id, parameters only apply to this mode
-        :param link_segment_external_id, parameters apply to this link segment 
+        :param mode_xml_id, parameters only apply to this mode
+        :param link_segment_xml_id, parameters apply to this link segment 
         """        
-        link_segment_counterpart = self._link_segments_instance.get_link_segment_by_external_id(link_segment_external_id, True)
-        link_segment_instance = LinkSegmentWrapper(link_segment_counterpart)
-        mode_counterpart = self._modes_instance.get_mode_by_external_id(mode_external_id, True)
-        self._java_counterpart.setParameters(link_segment_instance.java, mode_counterpart, alpha, beta)
+        mode_counterpart = self._modes_instance.get_by_xml_id(mode_xml_id)
+        layer_counterpart = self._layers_instance.get(mode_counterpart)
+        link_segments_counterpart = layer_counterpart.getLinkSegments()
+        link_segments_instance = LinkSegmentsWrapper(link_segments_counterpart)
+        
+        link_segment_counterpart = link_segments_instance.get_by_xml_id(link_segment_xml_id)        
+        self._java_counterpart.setParameters(link_segment_counterpart, mode_counterpart, alpha, beta)
 
 class MemoryOutputFormatterWrapper(OutputFormatterWrapper):
     """ Wrapper around the Java PlanItOutputFormatter class instance
@@ -425,21 +444,21 @@ class MemoryOutputFormatterWrapper(OutputFormatterWrapper):
         self._demands_instance = demands_instance
         self._network_instance = network_instance
                    
-    def iterator(self, mode_external_id, time_period_external_id, no_iterations, output_type):
+    def iterator(self, mode_xml_id: str, time_period_xml_id: str, no_iterations: int, output_type: OutputType):
         """Return the  wrapper for MemoryOutputIterator object for this MemoryOutputFormatter
-        :param mode_external_id the external Id of the current mode
-        :param time_period_external_id the external Id of the current time period
+        :param mode_xml_id the external Id of the current mode
+        :param time_period_xml_id the external Id of the current time period
         :param no_iterations the iteration the output iterator applies to
         :param output_type the output type for the current output
         :return the wrapper for the memory output iterator
         """
         time_periods_counterpart = self._demands_instance.field("timePeriods")
         time_periods = TimePeriodsWrapper(time_periods_counterpart)
-        time_period_counterpart = time_periods.get_time_period_by_external_id(time_period_external_id, True);
+        time_period_counterpart = time_periods.get_time_period_by_xml_id(time_period_xml_id);
         time_period = TimePeriodWrapper(time_period_counterpart)        
         modes_counterpart = self._network_instance.field("modes")
         modes = ModesWrapper(modes_counterpart)
-        mode_counterpart = modes.get_mode_by_external_id(mode_external_id, True)
+        mode_counterpart = modes.get_by_xml_id(mode_xml_id)
         mode = ModeWrapper(mode_counterpart)       
         output_type_instance = GatewayState.python_2_java_gateway.entry_point.createEnum(output_type.java_class_name(), output_type.value)
         memory_output_iterator_counterpart = self._java_counterpart.getIterator(mode.java, time_period.java, no_iterations, output_type_instance)
@@ -512,6 +531,6 @@ class PathOutputTypeConfigurationWrapper(OutputTypeConfigurationWrapper):
     def __init__(self, java_counterpart):
         super().__init__(java_counterpart)
         
-    def set_path_id_type(self,  route_id_type : RouteIdType):
-        route_id_type_instance = GatewayState.python_2_java_gateway.entry_point.createEnum(route_id_type.java_class_name(), route_id_type.value)
-        self._java_counterpart.setPathIdType(route_id_type_instance)
\ No newline at end of file
+    def set_path_id_type(self,  path_id_type : PathIdType):
+        path_id_type_instance = GatewayState.python_2_java_gateway.entry_point.createEnum(path_id_type.java_class_name(), path_id_type.value)
+        self._java_counterpart.setPathIdentificationType(path_id_type_instance)
\ No newline at end of file
